#!/usr/bin/env python3

"""
Pandoc filter to process raw latex tikz environments into images.
Assumes that pdflatex is in the path, and that the standalone
package is available.  Also assumes that ImageMagick's convert
is in the path. Images are put in the tikz-images directory.

CREDIT: Did I write this? Or downloaded it from somewhere? Oh the mystery.
"""

import os
import subprocess
import shutil
import sys
from subprocess import call
from tempfile import mkdtemp

from pandocfilters import (
    toJSONFilter,
    Para,
    Image,
    get_filename4code,
    get_extension,
    get_caption,
)


def print1(*args):
    print(*args, file=sys.stderr)


def tikz2image(tikz_src, filetype, outfile):
    tmpdir = mkdtemp()
    olddir = os.getcwd()
    os.chdir(tmpdir)
    f = open("tikz.tex", "w")
    f.write(
        """\\documentclass{standalone}
             \\usepackage{tikz}
             \\usetikzlibrary{positioning,calc,shapes,arrows,arrows.meta}
             \\begin{document}
             """
    )
    f.write(tikz_src)
    f.write("\n\\end{document}\n")
    f.close()
    call(["lualatex", "--shell-escape", "tikz.tex"], stdout=sys.stderr)
    os.chdir(olddir)
    if filetype == "pdf":
        shutil.copyfile(tmpdir + "/tikz.pdf", outfile + ".pdf")
    else:
        call(["convert", tmpdir + "/tikz.pdf", outfile + "." + filetype])
    shutil.rmtree(tmpdir)

def to_format(txt : str, fmt : str) -> str:
    cmd = f"pandoc -t {fmt}"
    s = subprocess.check_output(cmd.split(" "), input=txt, encoding='utf8')
    assert s, s
    return s

def _get_caption(item, fmt : str):
    captions, typef, keyvals = get_caption(item)
    #for caption in captions:
    #    if caption['c'] is None and caption['c']:
    #        continue
    #    caption['c'] = to_format(caption['c'], fmt)
    #    print1(caption)
    return captions, typef, keyvals


def tikz(key, value, format, _):
    if key == "CodeBlock":
        [[ident, classes, keyvals], code] = value
        if set(classes).intersection(set(["latex", "tex", "tikz"])):
            outfile = get_filename4code("tikz", code)
            filetype = get_extension(format, "png", html="png", latex="pdf")
            src = outfile + "." + filetype
            if not os.path.isfile(src):
                tikz2image(code, filetype, outfile)
                sys.stderr.write("Created image " + src + "\n")
            caption, typef, keyvals = _get_caption(keyvals, format)
            return Para([Image([ident, [], keyvals], caption, [src, typef])])

if __name__ == "__main__":
    toJSONFilter(tikz)



__author__ = "Dilawar Singh"
__email__ = "dilawar.s.rajput@gmail.com"

import math
import pickle
from pathlib import Path
import typing as T
import functools

from dilawar.logger import logger


def argmax(ls: list) -> int:
    """argmax: Returns the index i such that max(ls) == ls[i]"""
    _m, _mi = -math.inf, 0
    for i, v in enumerate(ls):
        if v > _m:
            _m = v
            _mi = i
    return _mi


def write_pickle(pklfile: Path, obj: T.Any):
    """Write pickle.

    Parameters
    ----------
    pklfile : Path
        pklfile
    obj : T.Any
        obj
    """
    with pklfile.open("wb") as f:
        pickle.dump(f, obj)

def load_pickle(pklfile: Path):
    """Write pickle.

    Parameters
    ----------
    pklfile : Path
        pklfile
    obj : T.Any
        obj
    """
    with pklfile.open("rb") as f:
        return pickle.load(f)


def run_if_not_pickled(pklfile: T.Union[str, Path]):
    def inner_decorator(func):
        @functools.wraps(func)
        def __inner(*args, **kwargs):
            p = Path(pklfile).resolve()
            if p.exists():
                try:
                    return load_pickle(p)
                except Exception as e:
                    logger.warning(f'Failed to load {p}/{e}')
                    p.unlink()

            x = func(*args, **kwargs)
            with p.open("wb") as f:
                pickle.dump(x, f)
            return x

        return __inner

    return inner_decorator

def flatten(listoflist : T.List[T.List[T.Any]]) -> T.List[T.Any]:
    import operator
    return functools.reduce(operator.iconcat, listoflist, [])

def test_argmax():
    import random

    a = [random.randint(-1000, 1000) for x in range(1000)]
    assert max(a) == a[argmax(a)]


@run_if_not_pickled("a.pickle")
def test_func_pickle():
    import numpy as np

    d = np.random.randint(0, 1000, 1000)
    return d

def test_flatten():
    a = [[1,1,1], [2], [[3], [-1]]]
    x = flatten(a)
    assert x == [1, 1, 1, 2, [3], [-1]]
    a = [[1], [2], [3]]
    x = flatten(a)
    assert x == [1, 2, 3]


def main():
    a = test_func_pickle()
    print(a.shape)
    test_flatten()


if __name__ == "__main__":
    main()



__author__ = "Dilawar Singh"
__email__ = "me@dilawars.me"

import wave
from pathlib import Path
import typing as T

def spectrogram(wavefile : str, nfft : int = 512, stride : int = 256):
    pass


def test_spectrogram():
    pass

if __name__ == "__main__":
    main()




import os
import datetime
from setuptools import setup

# __version__ will be evaluated.
with open("./dilawar/version.py") as f:
    exec(f.read())

version_ = __version__
stamp = datetime.datetime.now().strftime("%Y%m%d")
if os.environ.get("TRAVIS"):
    version_ += "-dist%s" % stamp

print("[INFO ] Packing %s" % version_)

with open("README.md") as f:
    readme = f.read()

install_requires = [
    "pint",
    "panflute",
    "pandocfilters",
    "colorama",
    "pandoc-imagine",
    "pypandoc",
]

setup(
    name="dilawar",
    version=version_,
    description="My personal utilities. See the README.md file.",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=["dilawar", "dilawar.pandoc"],
    package_dir={"dilawar": "dilawar"},
    scripts=["dilawar/bin/pandoc-gls.py", "dilawar/bin/pandoc-quantity.py"],
    package_data={
        # If any package contains *.txt or *.rst files, include them:
        "": ["*.tex", "*.css", "template/*"],
    },
    install_requires=install_requires,
    author="Dilawar Singh",
    author_email="dilawar.s.rajput@gmail.com",
    url="http://github.com/dilawar/dilawar",
    license="GPLv3",
)
